#!/usr/bin/perl
use strict;
use warnings;

#
# Creates a repository and a working copy that can be used to reproduce the
# environment in which the strange merge results appear.
#

use Cwd;

my $root = getcwd;

# my $root = "c:/mergeinfo";

# create the repository
qx(mkdir repo);
qx(svnadmin create repo/mergeinfo);

# check out the working copy
qx(mkdir wc);
qx(svn checkout file:///$root/repo/mergeinfo wc);

# create basic structure
chdir "wc";
my @folders = ('trunk', 'branches');
my @files = ('file_1', 'file_2', 'file_3');
foreach my $folder(@folders) {
  qx(mkdir $folder);
}

chdir "trunk";
foreach my $file(@files) {
  qx(echo "initial content" > $file);
}

@folders = ('folder_1', 'folder_2', 'folder_3');
@files = ('file_1', 'file_2', 'file_3');
foreach my $folder(@folders) {
  qx(mkdir $folder);
  chdir $folder;

  foreach my $file(@files) {
    qx(echo "initial content" > $file);
  }
  
  chdir "..";
}

# add and commit initial data
chdir "..";
qx(svn add trunk);
qx(svn add branches);
qx(svn commit -m "initial checkin");  #r1

# create a branch and check it out as sparse working copy (r2)
qx(svn copy file:///$root/repo/mergeinfo/trunk file:///$root/repo/mergeinfo/branches/feat_1 -m "creating branch feat_1");

chdir "branches";
qx(svn up --depth immediates);
chdir "feat_1";
qx(svn up --set-depth immediates);

# do some work and commit it
qx(echo "modified content" > file_1);
qx(svn commit -m "modified file_1");  #r3

# do something important on trunk and commit it
chdir "..";
chdir "..";
chdir "trunk";
qx(echo "important fix made on trunk" > file_2);
qx(svn commit -m "fix: file_2");      #r4

# cherry pick the important change
chdir "..";
chdir "branches";
chdir "feat_1";
qx(svn merge -c 4 file:///$root/repo/mergeinfo/trunk");
qx(svn up);
qx(svn commit -m "merged in revision 4 from trunk, contained important fix"); #r5

# do some more work and commit it
qx(echo "modified content" > file_3);
qx(svn commit -m "modified file_1");  #r6

# go back to trunk and make it a sparse checkout
chdir "..";
chdir "..";
chdir "trunk";
qx(svn up --set-depth immediates);

# merge in the branch and commit the result (now the folders of the trunk contain explicit, non-inheritable merge infos)
qx(svn merge file:///$root/repo/mergeinfo/branches/feat_1");
qx(svn commit -m "merged in branch feat_1");  #r7

# create cleanup branch and check it out (r8)
qx(svn copy file:///$root/repo/mergeinfo/trunk file:///$root/repo/mergeinfo/branches/cleanup -m "creating branch cleanup");
chdir "..";
chdir "branches";
qx(svn up);
